<?php
// wbb imports
require_once(WBB_DIR.'lib/data/board/BoardEditor.class.php');

// wcf imports
require_once(WCF_DIR.'lib/acp/form/ACPForm.class.php');
require_once(WCF_DIR.'lib/data/user/User.class.php');
require_once(WCF_DIR.'lib/data/user/group/Group.class.php');
require_once(WCF_DIR.'lib/system/session/Session.class.php');
require_once(WCF_DIR.'lib/system/style/Style.class.php');

/**
 * Shows the board add form.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2007 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb.acp.form
 */
class BoardAddForm extends ACPForm {
	public $templateName = 'boardAdd';
	public $activeMenuItem = 'wbb.acp.menu.link.content.board.add';
	public $neededPermissions = 'admin.board.canAddBoard';
	public $activeTabMenuItem = 'data';
	
	public $board;
	public $boardType = 0;
	public $parentID = 0;
	public $position = '';
	public $title = '';
	public $description = '';
	public $allowDescriptionHtml = 0;
	public $sortField = '';
	public $sortOrder = '';
	public $closed = 0;
	public $countUserPosts = 1;
	public $invisible = 0;
	public $permissions = array();
	public $permissionSettings = array();
	public $boardOptions = array();
	public $additionalFields = array();
	
	/**
	 * @see Page::readParameters()
	 */
	public function readParameters() {
		parent::readParameters();
		
		$this->readPermissionSettings();
	}
	
	/**
	 * @see Form::readFormParameters()
	 */
	public function readFormParameters() {
		parent::readFormParameters();
		
		$this->closed = $this->countUserPosts = $this->invisible = $this->allowDescriptionHtml = 0;
		
		if (isset($_POST['boardType'])) $this->boardType = intval($_POST['boardType']);
		if (isset($_POST['parentID'])) $this->parentID = intval($_POST['parentID']);
		if (!empty($_POST['position'])) $this->position = intval($_POST['position']);
		if (isset($_POST['title'])) $this->title = StringUtil::trim($_POST['title']);
		if (isset($_POST['description'])) $this->description = StringUtil::trim($_POST['description']);
		if (isset($_POST['allowDescriptionHtml'])) $this->allowDescriptionHtml = intval($_POST['allowDescriptionHtml']);
		if (isset($_POST['sortField'])) $this->sortField = $_POST['sortField'];
		if (isset($_POST['sortOrder'])) $this->sortOrder = $_POST['sortOrder'];
		if (isset($_POST['closed'])) $this->closed = intval($_POST['closed']);
		if (isset($_POST['countUserPosts'])) $this->countUserPosts = intval($_POST['countUserPosts']);
		if (isset($_POST['invisible'])) $this->invisible = intval($_POST['invisible']);
		if (isset($_POST['activeTabMenuItem'])) $this->activeTabMenuItem = $_POST['activeTabMenuItem'];
		
		// permissions
		if (isset($_POST['permission']) && is_array($_POST['permission'])) $this->permissions = $_POST['permission'];
	}
	
	/**
	 * Validates the given permissions.
	 */
	public function validatePermissions($permissions, $validSettings) {
		foreach ($permissions as $permission) {
			// type
			if (!isset($permission['type']) || ($permission['type'] != 'user' && $permission['type'] != 'group')) {
				throw new UserInputException();
			}
			
			// id
			if (!isset($permission['id'])) {
				throw new UserInputException();
			}
			if ($permission['type'] == 'user') {
				$user = new User(intval($permission['id']));
				if (!$user->userID) throw new UserInputException();
			}
			else {
				$group = new Group(intval($permission['id']));
				if (!$group->groupID) throw new UserInputException();
			}
			
			// settings
			if (!isset($permission['settings']) || !is_array($permission['settings'])) {
				throw new UserInputException();
			}
			// find invalid settings
			foreach ($permission['settings'] as $key => $value) {
				if (!isset($validSettings[$key]) || ($value != -1 && $value != 0 && $value =! 1)) {
					throw new UserInputException();
				}
			}
			// find missing settings
			foreach ($validSettings as $key => $value) {
				if (!isset($permission['settings'][$key])) {
					throw new UserInputException();
				}
			}
		}
	}
	
	/**
	 * @see Form::validate()
	 */
	public function validate() {
		// validate permissions
		$this->validatePermissions($this->permissions, array_flip($this->permissionSettings));
		
		parent::validate();
		
		// board type
		if ($this->boardType < 0 || $this->boardType > 1) {
			throw new UserInputException('boardType', 'invalid');
		}
		
		// parent id
		$this->validateParentID();
		
		// position
		/*if (!$this->position) {
			throw new UserInputException('position');
		}*/
		
		// title
		if (empty($this->title)) {
			throw new UserInputException('title');
		}
		
		// sortField
		switch ($this->sortField) {
			case '': case 'topic': case 'username': 
			case 'time': case 'replies': case 'views': case 'lastPostTime': break;
			default: throw new UserInputException('sortField', 'invalid');
		}
		
		// sortOrder
		switch ($this->sortOrder) {
			case '': case 'ASC': case 'DESC': break;
			default: throw new UserInputException('sortOrder', 'invalid');
		}
	}
	
	/**
	 * Validates the given parent id.
	 */
	protected function validateParentID() {
		if ($this->parentID) {
			require_once(WCF_DIR.'lib/system/exception/IllegalLinkException.class.php');
			try {
				Board::getBoard($this->parentID);
			}
			catch (IllegalLinkException $e) {
				throw new UserInputException('parentID', 'invalid');
			}
		}
	}
	
	/**
	 * @see Form::save()
	 */
	public function save() {
		parent::save();
		
		// save board
		if (WCF::getUser()->getPermission('admin.board.canEditBoard')) {
			$this->board = BoardEditor::create($this->parentID, ($this->position ? $this->position : null), $this->title, $this->description,
			$this->boardType, TIME_NOW, $this->sortField, $this->sortOrder, $this->closed, $this->countUserPosts,
			$this->invisible, $this->allowDescriptionHtml, $this->additionalFields);
		}
		
		// save permissions
		if (WCF::getUser()->getPermission('admin.board.canEditPermissions')) {
			$this->savePermissions();
		}
		
		// reset cache
		$this->resetCache();
		$this->saved();
		
		// reset values
		$this->boardType = $this->parentID = $this->closed = $this->invisible = $this->allowDescriptionHtml = 0;
		$this->countUserPosts = 1;
		$this->position = $this->title = $this->description = $this->sortField = $this->sortOrder = '';
		$this->permissions = array();
		
		// show success message
		WCF::getTPL()->assign(array(
			'board' => $this->board,
			'success' => true
		));
	}
	
	/**
	 * Resets the board cache after changes.
	 */
	protected function resetCache() {
		Board::resetCache();
		
		// reset sessions
		Session::resetSessions();
	}
	
	/**
	 * Saves user and group permissions.
	 */
	public function savePermissions() {
		// create inserts
		$userInserts = $groupInserts = '';
		foreach ($this->permissions as $key => $permission) {
			// skip default values
			$noDefaultValue = false;
			foreach ($permission['settings'] as $value) {
				if ($value != -1) $noDefaultValue = true;
			}
			if (!$noDefaultValue) {
				unset($this->permissions[$key]);
				continue;
			}
			
			if ($permission['type'] == 'user') {
				if (!empty($userInserts)) $userInserts .= ',';
				$userInserts .= '('.$this->board->boardID.',
						 '.intval($permission['id']).',
						 '.(implode(', ', ArrayUtil::toIntegerArray($permission['settings']))).')';
			
			}
			else {
				if (!empty($groupInserts)) $groupInserts .= ',';
				$groupInserts .= '('.$this->board->boardID.',
						 '.intval($permission['id']).',
						 '.(implode(', ', ArrayUtil::toIntegerArray($permission['settings']))).')';
			}
		}
	
		if (!empty($userInserts)) {
			$sql = "INSERT INTO	wbb".WBB_N."_board_to_user
						(boardID, userID, ".implode(', ', $this->permissionSettings).")
				VALUES		".$userInserts;
			WCF::getDB()->sendQuery($sql);
		}
		
		if (!empty($groupInserts)) {
			$sql = "INSERT INTO	wbb".WBB_N."_board_to_group
						(boardID, groupID, ".implode(', ', $this->permissionSettings).")
				VALUES		".$groupInserts;
			WCF::getDB()->sendQuery($sql);
		}
	}
	
	/**
	 * @see Page::readData()
	 */
	public function readData() {
		parent::readData();
		
		$this->readBoardOptions();
	}
	
	/**
	 * @see Page::assignVariables()
	 */
	public function assignVariables() {
		parent::assignVariables();
		
		WCF::getTPL()->assign(array(
			'boardType' => $this->boardType,
			'parentID' => $this->parentID,
			'position' => $this->position,
			'title' => $this->title,
			'description' => $this->description,
			'allowDescriptionHtml' => $this->allowDescriptionHtml,
			'sortField' => $this->sortField,
			'sortOrder' => $this->sortOrder,
			'closed' => $this->closed,
			'countUserPosts' => $this->countUserPosts,
			'invisible' => $this->invisible,
			'boardOptions' => $this->boardOptions,
			'permissions' => $this->permissions,
			'permissionSettings' => $this->permissionSettings,
			'action' => 'add',
			'activeTabMenuItem' => $this->activeTabMenuItem
		));
	}
	
	/**
	 * Gets available permission settings.
	 */
	protected function readPermissionSettings() {
		$this->permissionSettings = array(
			'canViewBoard', 'canEnterBoard', 'canReadThread', 'canStartThread',
			'canReplyThread', 'canVotePoll', 'canStartPoll', 'canUploadAttachment',
			'canDownloadAttachment', 'canEditOwnPost', 'canDeleteOwnPost'
		);
	}
	
	/**
	 * Gets a list of available parent boards.
	 */
	protected function readBoardOptions() {
		$this->boardOptions = Board::getBoardSelect(array(), false, true);
	}
}
?>