<?php
require_once(WCF_DIR.'lib/page/AbstractPage.class.php');
require_once(WCF_DIR.'lib/acp/package/PackageInstallationQueue.class.php');
require_once(WCF_DIR.'lib/data/feed/FeedReaderSource.class.php');
require_once(WCF_DIR.'lib/acp/package/ServerUpdate.class.php');

/**
 * Shows the welcome page in wbb admin control panel.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2007 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb.acp.page
 */
class IndexPage extends AbstractPage {
	public $templateName = 'index';
	
	public $os = '', $webserver = '', $sqlVersion = '', $sqlType = '', $load = '';
	public $stat = array();
	public $news = array();
	public $updates = array();
	
	/**
	 * @see Page::readData()
	 */
	public function readData() {
		parent::readData();
		
		$this->os = PHP_OS;
		if (isset($_SERVER['SERVER_SOFTWARE'])) $this->webserver = $_SERVER['SERVER_SOFTWARE'];
		$this->sqlVersion = WCF::getDB()->getVersion();
		$this->sqlType = WCF::getDB()->getDBType();
		$this->readLoad();
		$this->readStat();
		
		// updates
		if (WCF::getUser()->getPermission('admin.system.package.canUpdatePackage')) {
			require_once(WCF_DIR.'lib/acp/package/update/PackageUpdate.class.php');
			$this->updates = PackageUpdate::getAvailableUpdates();
			
			// kick wbb 3.0 updates
			foreach ($this->updates as $packageID => $package) {
				if ($package['package'] == 'com.woltlab.wbb') {
					foreach ($package['versions'] as $version => $packageVersion) {
						if (Package::compareVersion($version, '3.0.0 Beta 1', '>=')) {
							unset($this->updates[$packageID]['versions'][$version]);
						}
					}
					
					if (!count($this->updates[$packageID]['versions'])) {
						unset($this->updates[$packageID]);
					}
					else {
						$this->updates[$packageID]['version'] = end($this->updates[$packageID]['versions']);
					}
				}
			}
		}
			
		// news
		$this->news = FeedReaderSource::getEntries(5);
		foreach ($this->news as $key => $news) {
			$this->news[$key]['description'] = preg_replace('/href="(.*?)"/e', '\'href="'.RELATIVE_WCF_DIR.'acp/dereferrer.php?url=\'.rawurlencode(\'$1\').\'" class="externalURL"\'', $news['description']);
		}
	}
	
	/**
	 * @see Page::assignVariables()
	 */
	public function assignVariables() {
		parent::assignVariables();
		
		WCF::getTPL()->assign(array(
			'os' => $this->os,
			'webserver' => $this->webserver,
			'sqlVersion' => $this->sqlVersion,
			'sqlType' => $this->sqlType,
			'load' => $this->load,
			'news' => $this->news,
			'updates' => $this->updates,
			'dbName' => WCF::getDB()->getDatabaseName()
		));
		WCF::getTPL()->assign($this->stat);
	}
	
	/**
	 * Gets a list of simple statistics.
	 */
	protected function readStat() {
		$installationAge = (TIME_NOW - INSTALL_DATE) / 86400;
		if ($installationAge < 1) $installationAge = 1;
		
		// members
		$sql = "SELECT	COUNT(*) AS members
			FROM	wcf".WCF_N."_user";
		$row = WCF::getDB()->getFirstRow($sql);
		$this->stat['members'] = $row['members'];
		
		// threads
		$sql = "SELECT	COUNT(*) AS threads
			FROM	wbb".WBB_N."_thread";
		$row = WCF::getDB()->getFirstRow($sql);
		$this->stat['threads'] = $row['threads'];
		$this->stat['threadsPerDay'] = $row['threads'] / $installationAge;
		
		// posts
		$sql = "SELECT	COUNT(*) AS posts
			FROM	wbb".WBB_N."_post";
		$row = WCF::getDB()->getFirstRow($sql);
		$this->stat['posts'] = $row['posts'];
		$this->stat['postsPerDay'] = $row['posts'] / $installationAge;
		
		// database entries and size
		$this->stat['databaseSize'] = 0;
		$this->stat['databaseEntries'] = 0;
		$sql = "SHOW TABLE STATUS";
		
		$result = WCF::getDB()->sendQuery($sql);
		while($row = WCF::getDB()->fetchArray($result)) {
			$this->stat['databaseSize'] += $row['Data_length'] + $row['Index_length'];
			$this->stat['databaseEntries'] += $row['Rows'];
		}
	}
	
	/**
	 * Gets the current server load.
	 */
	protected function readLoad() {
		if ($uptime = @exec("uptime")) {
			if (preg_match("/averages?: ([0-9\.]+,?[\s]+[0-9\.]+,?[\s]+[0-9\.]+)/", $uptime, $match)) {
				$this->load = $match[1];
			}
		}
	}
}
?>