<?php
// wcf imports
require_once(WCF_DIR.'lib/page/util/menu/HeaderMenuContainer.class.php');
require_once(WCF_DIR.'lib/page/util/menu/UserCPMenuContainer.class.php');
require_once(WCF_DIR.'lib/page/util/menu/UserProfileMenuContainer.class.php');
require_once(WCF_DIR.'lib/system/style/Style.class.php');

/**
 * This class extends the main WCF class by forum specific functions.
 *
 * @package	com.woltlab.wbb.system
 * @author	Marcel Werk
 * @copyright	2001-2007 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 */
class WBBCore extends WCF implements HeaderMenuContainer, UserCPMenuContainer, UserProfileMenuContainer {
	protected static $headerMenuObj = null;
	protected static $userCPMenuObj = null;
	protected static $userProfileMenuObj = null;

	/**
	 * @see WCF::initTPL()
	 */
	protected function initTPL() {
		// init style to get template pack id
		$this->initStyle();
		
		global $packageDirs;
 		require_once(WCF_DIR.'lib/system/template/StructuredTemplate.class.php');
		self::$tplObj = new StructuredTemplate(self::getStyle()->templatePackID, self::getLanguage()->getLanguageID(), ArrayUtil::appendSuffix($packageDirs, 'templates/'));
		$this->assignDefaultTemplateVariables();
		
		// init cronjobs
		$this->initCronjobs();
		
		// user ban
		if (self::getUser()->banned && (!isset($_REQUEST['page']) || $_REQUEST['page'] != 'LegalNotice')) {
			require_once(WCF_DIR.'lib/system/exception/PermissionDeniedException.class.php');
			throw new PermissionDeniedException();
		}
	}
	
	/**
	 * Initialises the cronjobs.
	 */
	protected function initCronjobs() {
		self::getTPL()->assign('executeCronjobs', WCF::getCache()->get('cronjobs-'.PACKAGE_ID, 'nextExec') < TIME_NOW);
	}
	
	/**
	 * @see WCF::loadDefaultCacheResources()
	 */
	protected function loadDefaultCacheResources() {
		parent::loadDefaultCacheResources();
		$this->loadDefaultWBBCacheResources();
	}
	
	/**
	 * Loads default cache resources of burning board.
	 * Can be called statically from other applications or plugins.
	 */
	public static function loadDefaultWBBCacheResources() {
		WCF::getCache()->addResource('board', WBB_DIR.'cache/cache.board.php', WBB_DIR.'lib/system/cache/CacheBuilderBoard.class.php');
		WCF::getCache()->addResource('boardData', WBB_DIR.'cache/cache.boardData.php', WBB_DIR.'lib/system/cache/CacheBuilderBoardData.class.php', 0, 300);
		WCF::getCache()->addResource('stat', WBB_DIR.'cache/cache.stat.php', WBB_DIR.'lib/system/cache/CacheBuilderStat.class.php', 0, 300);
		WCF::getCache()->addResource('bbcodes', WCF_DIR.'cache/cache.bbcodes.php', WCF_DIR.'lib/system/cache/CacheBuilderBBCodes.class.php');
		WCF::getCache()->addResource('smilies', WCF_DIR.'cache/cache.smilies.php', WCF_DIR.'lib/system/cache/CacheBuilderSmilies.class.php');
		WCF::getCache()->addResource('cronjobs-'.PACKAGE_ID, WCF_DIR.'cache/cache.cronjobs-'.PACKAGE_ID.'.php', WCF_DIR.'lib/system/cache/CacheBuilderCronjobs.class.php');
		WCF::getCache()->addResource('help-'.PACKAGE_ID, WCF_DIR.'cache/cache.help-'.PACKAGE_ID.'.php', WCF_DIR.'lib/system/cache/CacheBuilderHelp.class.php');
	}
	
	/**
	 * Initialises the page header menu.
	 */
	protected static function initHeaderMenu() {
		require_once(WCF_DIR.'lib/page/util/menu/HeaderMenu.class.php');
		self::$headerMenuObj = new HeaderMenu();
		if (HeaderMenu::getActiveMenuItem() == '') HeaderMenu::setActiveMenuItem('wbb.header.menu.board');
	}
	
	/**
	 * Initialises the user cp menu.
	 */
	protected static function initUserCPMenu() {
		require_once(WCF_DIR.'lib/page/util/menu/UserCPMenu.class.php');
		self::$userCPMenuObj = UserCPMenu::getInstance();
	}
	
	/**
	 * Initialises the user profile menu.
	 */
	protected static function initUserProfileMenu() {
		require_once(WCF_DIR.'lib/page/util/menu/UserProfileMenu.class.php');
		self::$userProfileMenuObj = UserProfileMenu::getInstance();
	}
	
	/**
	 * @see WCF::getOptionsFilename()
	 */
	protected function getOptionsFilename() {
		return WBB_DIR.'options.inc.php';
	}
	
	/**
	 * Initialises the style system.
	 */
	protected function initStyle() {
		Style::changeStyle(self::getSession()->getStyleID());
	}
	
	/**
	 * Changes the active style.
	 * 
	 * @param	integer		$styleID
	 * @deprecated
	 */
	public static final function changeStyle($styleID) {
		return Style::changeStyle($styleID);
	}
	
	/**
	 * @see HeaderMenuContainer::getHeaderMenu()
	 */
	public static final function getHeaderMenu() {
		if (self::$headerMenuObj === null) {
			self::initHeaderMenu();
		}
		
		return self::$headerMenuObj;
	}
	
	/**
	 * @see UserCPMenuContainer::getUserCPMenu()
	 */
	public static final function getUserCPMenu() {
		if (self::$userCPMenuObj === null) {
			self::initUserCPMenu();
		}
		
		return self::$userCPMenuObj;
	}
	
	/**
	 * @see UserProfileMenuContainer::getUserProfileMenu()
	 */
	public static final function getUserProfileMenu() {
		if (self::$userProfileMenuObj === null) {
			self::initUserProfileMenu();
		}
		
		return self::$userProfileMenuObj;
	}
	
	/**
	 * Returns the active style object.
	 * 
	 * @return	Style
	 */
	public static final function getStyle() {
		return Style::getStyle();
	}
	
	/**
	 * @see WCF::initSession()
	 */
	protected function initSession() {
		// start session
		require_once(WBB_DIR.'lib/system/session/WBBSessionFactory.class.php');
		$factory = new WBBSessionFactory();
		self::$sessionObj = $factory->get();
		self::$userObj = self::getSession()->getUser();
	}
	
	/**
	 * @see	WCF::assignDefaultTemplateVariables()
	 */
	protected function assignDefaultTemplateVariables() {
		parent::assignDefaultTemplateVariables();
		self::getTPL()->assign('timezone', DateUtil::getTimezone());
	}
}
?>