<?php
require_once(WBB_DIR.'lib/system/cronjob/LastActivityCronjob.class.php');

/**
 * Cronjob for a hourly system cleanup.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2007 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb.system.cronjob
 */
class CleanupCronjob extends LastActivityCronjob {
	/**
	 * @see Cronjob::execute()
	 */
	public function execute($data) {
		parent::execute($data);
		
		// delete old sessions
		$sql = "DELETE FROM	wcf".WCF_N."_session
			WHERE		lastActivityTime < ".(TIME_NOW - SESSION_TIMEOUT);
		WCF::getDB()->registerShutdownUpdate($sql);
		
		// delete old acp sessions
		$sql = "DELETE FROM	wcf".WCF_N."_acp_session
			WHERE		lastActivityTime < ".(TIME_NOW - SESSION_TIMEOUT);
		WCF::getDB()->registerShutdownUpdate($sql);
		
		// delete old captchas
		$sql = "DELETE FROM	wcf".WCF_N."_captcha
			WHERE		captchaDate < ".(TIME_NOW - 3600);
		WCF::getDB()->registerShutdownUpdate($sql);
		
		// delete post cache
		$sql = "DELETE FROM	wbb".WBB_N."_post_cache
			WHERE		threadID IN (
						SELECT	threadID
						FROM	wbb".WBB_N."_thread
						WHERE	lastPostTime < ".(TIME_NOW - 86400 * 7)."
					)";
		WCF::getDB()->registerShutdownUpdate($sql);
		
		// delete searches
		$sql = "DELETE FROM	wcf".WCF_N."_search
			WHERE		searchDate < ".(TIME_NOW - 7200);
		WCF::getDB()->registerShutdownUpdate($sql);
		
		// delete orphaned attachments
		$sql = "SELECT	attachmentID
			FROM	wcf".WCF_N."_attachment
			WHERE	messageID = 0
				AND uploadTime < ".(TIME_NOW - 43200);
		$result = WCF::getDB()->sendQuery($sql);
		if (WCF::getDB()->countRows($result) > 0) {
			require_once(WCF_DIR.'lib/data/message/attachment/AttachmentsEditor.class.php');
			$attachmentIDs = '';
			while ($row = WCF::getDB()->fetchArray($result)) {
				if (!empty($attachmentIDs)) $attachmentIDs .= ',';
				$attachmentIDs .= $row['attachmentID'];
				
				// delete files
				AttachmentsEditor::deleteFile($row['attachmentID']);
			}
			
			if (!empty($attachmentIDs)) {
				$sql = "DELETE FROM	wcf".WCF_N."_attachment
					WHERE		attachmentID IN (".$attachmentIDs.")";
				WCF::getDB()->registerShutdownUpdate($sql);
			}
		}
		
		// delete post / pm hashes
		$sql = "DELETE FROM	wbb".WBB_N."_post_hash
			WHERE		time < ".(TIME_NOW - 3600);
		WCF::getDB()->registerShutdownUpdate($sql);
		$sql = "DELETE FROM	wcf".WCF_N."_pm_hash
			WHERE		time < ".(TIME_NOW - 3600);
		WCF::getDB()->registerShutdownUpdate($sql);
		
		// delete bad user data
		$sql = "DELETE FROM	wbb".WBB_N."_user
			WHERE		userID NOT IN (
						SELECT	userID
						FROM	wcf".WCF_N."_user
					)";
		WCF::getDB()->registerShutdownUpdate($sql);
	}
}
?>